@@ -4,10 +4,10 @@
 # This module is part of GitPython and is released under
 # the BSD License: http://www.opensource.org/licenses/bsd-license.php
 
+# flake8: noqa
 import inspect
-from mock import *
-from asserts import *
-from helper import *
+from .asserts import *
+from .helper import *
 
 __all__ = [name for name, obj in locals().items()
-            if not (name.startswith('_') or inspect.ismodule(obj))]
+           if not (name.startswith('_') or inspect.ismodule(obj))]
