@@ -1,8 +1,10 @@
+# coding: utf-8
 # test_diff.py
 # Copyright (C) 2008, 2009 Michael Trier (mtrier@gmail.com) and contributors
 #
 # This module is part of GitPython and is released under
 # the BSD License: http://www.opensource.org/licenses/bsd-license.php
+import os
 
 from git.test.lib import (
     TestBase,
@@ -13,9 +15,14 @@ from git.test.lib import (
 
 )
 
+from gitdb.test.lib import with_rw_directory
+
 from git import (
+    Repo,
+    GitCommandError,
     Diff,
-    DiffIndex
+    DiffIndex,
+    NULL_TREE,
 )
 
 
@@ -36,13 +43,40 @@ class TestDiff(TestBase):
         # END for each diff
         return diffs
 
+    @with_rw_directory
+    def test_diff_with_staged_file(self, rw_dir):
+        # SETUP INDEX WITH MULTIPLE STAGES
+        r = Repo.init(rw_dir)
+        fp = os.path.join(rw_dir, 'hello.txt')
+        with open(fp, 'w') as fs:
+            fs.write("hello world")
+        r.git.add(fp)
+        r.git.commit(message="init")
+
+        with open(fp, 'w') as fs:
+            fs.write("Hola Mundo")
+        r.git.commit(all=True, message="change on master")
+
+        r.git.checkout('HEAD~1', b='topic')
+        with open(fp, 'w') as fs:
+            fs.write("Hallo Welt")
+        r.git.commit(all=True, message="change on topic branch")
+
+        # there must be a merge-conflict
+        self.failUnlessRaises(GitCommandError, r.git.cherry_pick, 'master')
+
+        # Now do the actual testing - this should just work
+        assert len(r.index.diff(None)) == 2
+
+        assert len(r.index.diff(None, create_patch=True)) == 0, "This should work, but doesn't right now ... it's OK"
+
     def test_list_from_string_new_mode(self):
         output = StringProcessAdapter(fixture('diff_new_mode'))
         diffs = Diff._index_from_patch_format(self.rorepo, output.stdout)
         self._assert_diff_format(diffs)
 
         assert_equal(1, len(diffs))
-        assert_equal(10, len(diffs[0].diff.splitlines()))
+        assert_equal(8, len(diffs[0].diff.splitlines()))
 
     def test_diff_with_rename(self):
         output = StringProcessAdapter(fixture('diff_rename'))
@@ -53,8 +87,9 @@ class TestDiff(TestBase):
 
         diff = diffs[0]
         assert_true(diff.renamed)
-        assert_equal(diff.rename_from, 'AUTHORS')
-        assert_equal(diff.rename_to, 'CONTRIBUTORS')
+        assert_equal(diff.rename_from, u'Jérôme')
+        assert_equal(diff.rename_to, u'müller')
+        assert isinstance(str(diff), str)
 
         output = StringProcessAdapter(fixture('diff_rename_raw'))
         diffs = Diff._index_from_raw_format(self.rorepo, output.stdout)
@@ -81,25 +116,78 @@ class TestDiff(TestBase):
         res = Diff._index_from_patch_format(None, output.stdout)
         assert len(res) == 6
         for dr in res:
-            assert dr.diff
+            assert dr.diff.startswith(b'@@')
             assert str(dr), "Diff to string conversion should be possible"
         # end for each diff
 
         dr = res[3]
         assert dr.diff.endswith(b"+Binary files a/rps and b/rps differ\n")
 
+    def test_diff_index_raw_format(self):
+        output = StringProcessAdapter(fixture('diff_index_raw'))
+        res = Diff._index_from_raw_format(None, output.stdout)
+        assert res[0].deleted_file
+        assert res[0].b_path == ''
+
+    def test_diff_initial_commit(self):
+        initial_commit = self.rorepo.commit('33ebe7acec14b25c5f84f35a664803fcab2f7781')
+
+        # Without creating a patch...
+        diff_index = initial_commit.diff(NULL_TREE)
+        assert diff_index[0].b_path == 'CHANGES'
+        assert diff_index[0].new_file
+        assert diff_index[0].diff == ''
+
+        # ...and with creating a patch
+        diff_index = initial_commit.diff(NULL_TREE, create_patch=True)
+        assert diff_index[0].a_path is None, repr(diff_index[0].a_path)
+        assert diff_index[0].b_path == 'CHANGES', repr(diff_index[0].b_path)
+        assert diff_index[0].new_file
+        assert diff_index[0].diff == fixture('diff_initial')
+
+    def test_diff_unsafe_paths(self):
+        output = StringProcessAdapter(fixture('diff_patch_unsafe_paths'))
+        res = Diff._index_from_patch_format(None, output.stdout)
+
+        # The "Additions"
+        self.assertEqual(res[0].b_path, u'path/ starting with a space')
+        self.assertEqual(res[1].b_path, u'path/"with-quotes"')
+        self.assertEqual(res[2].b_path, u"path/'with-single-quotes'")
+        self.assertEqual(res[3].b_path, u'path/ending in a space ')
+        self.assertEqual(res[4].b_path, u'path/with\ttab')
+        self.assertEqual(res[5].b_path, u'path/with\nnewline')
+        self.assertEqual(res[6].b_path, u'path/with spaces')
+        self.assertEqual(res[7].b_path, u'path/with-question-mark?')
+        self.assertEqual(res[8].b_path, u'path/¯\\_(ツ)_|¯')
+
+        # The "Moves"
+        # NOTE: The path prefixes a/ and b/ here are legit!  We're actually
+        # verifying that it's not "a/a/" that shows up, see the fixture data.
+        self.assertEqual(res[9].a_path, u'a/with spaces')       # NOTE: path a/ here legit!
+        self.assertEqual(res[9].b_path, u'b/with some spaces')  # NOTE: path b/ here legit!
+        self.assertEqual(res[10].a_path, u'a/ending in a space ')
+        self.assertEqual(res[10].b_path, u'b/ending with space ')
+        self.assertEqual(res[11].a_path, u'a/"with-quotes"')
+        self.assertEqual(res[11].b_path, u'b/"with even more quotes"')
+
     def test_diff_patch_format(self):
         # test all of the 'old' format diffs for completness - it should at least
         # be able to deal with it
         fixtures = ("diff_2", "diff_2f", "diff_f", "diff_i", "diff_mode_only",
                     "diff_new_mode", "diff_numstat", "diff_p", "diff_rename",
-                    "diff_tree_numstat_root")
+                    "diff_tree_numstat_root", "diff_patch_unsafe_paths")
 
         for fixture_name in fixtures:
             diff_proc = StringProcessAdapter(fixture(fixture_name))
             Diff._index_from_patch_format(self.rorepo, diff_proc.stdout)
         # END for each fixture
 
+    def test_diff_with_spaces(self):
+        data = StringProcessAdapter(fixture('diff_file_with_spaces'))
+        diff_index = Diff._index_from_patch_format(self.rorepo, data.stdout)
+        assert diff_index[0].a_path is None, repr(diff_index[0].a_path)
+        assert diff_index[0].b_path == u'file with spaces', repr(diff_index[0].b_path)
+
     def test_diff_interface(self):
         # test a few variations of the main diff routine
         assertion_map = dict()
@@ -109,7 +197,7 @@ class TestDiff(TestBase):
                 diff_item = commit.tree
             # END use tree every second item
 
-            for other in (None, commit.Index, commit.parents[0]):
+            for other in (None, NULL_TREE, commit.Index, commit.parents[0]):
                 for paths in (None, "CHANGES", ("CHANGES", "lib")):
                     for create_patch in range(2):
                         diff_index = diff_item.diff(other=other, paths=paths, create_patch=create_patch)
